<?php

namespace QUI\TrustedShops;

use DateTime;
use DateTimeInterface;
use QUI;
use QUI\ERP\Accounting\Article;
use QUI\ERP\Order\AbstractOrder;
use QUI\Exception;
use QUI\Projects\Project;

use function curl_close;
use function curl_error;
use function curl_exec;
use function curl_init;
use function curl_setopt;
use function http_build_query;
use function is_array;
use function json_decode;
use function json_encode;

use const CURLOPT_HEADER;
use const CURLOPT_HTTPHEADER;
use const CURLOPT_POSTFIELDS;
use const CURLOPT_RETURNTRANSFER;
use const CURLOPT_SSL_VERIFYPEER;
use const CURLOPT_TIMEOUT;
use const CURLOPT_URL;
use const HOST;

class TrustedShops
{
    /**
     * API:
     * https://developers.etrusted.com/events-api/events-api.html
     *
     * @param AbstractOrder $Order
     * @param Project|null $Project
     *
     * @return void
     *
     * @throws Exception
     */
    public static function sendOrder(
        AbstractOrder $Order,
        ?Project $Project = null
    ): void {
        if ($Project === null) {
            $Project = QUI::getProjectManager()->get();
        }

        $send = $Order->getDataEntry('quiqqer-trustedshops-send-events');

        if ($send) {
            return;
        }

        $sendOrder = (int)$Project->getConfig('trustedshops.api.sendOrder');
        $sendProducts = (int)$Project->getConfig('trustedshops.api.sendProducts');
        $channel = self::getChannel($Project);

        if (empty($sendOrder)) {
            return;
        }

        $token = self::getToken($Project);

        if (empty($token)) {
            return;
        }

        // customer stuff
        $Customer = $Order->getCustomer();
        $Address = $Customer->getAddress();
        $phone = $Address->getPhone();

        if (empty($phone)) {
            $phone = $Address->getMobile();
        }

        $email = '';
        $mails = $Address->getMailList();

        if (count($mails)) {
            $email = $mails[0];
        }

        // products
        $products = [];

        if (!empty($sendProducts)) {
            $Articles = $Order->getArticles();

            /* @var $Article Article */
            foreach ($Articles as $Article) {
                $productId = $Article->getId();

                try {
                    $Product = QUI\ERP\Products\Handler\Products::getProduct($productId);
                } catch (Exception) {
                    continue;
                }

                $products[] = [
                    "gtin" => $Article->getGTIN(),
                    "imageUrl" => HOST . $Article->getImage()->getSizeCacheUrl(),
                    "name" => $Article->getTitle(),
                    "sku" => $Article->getArticleNo(),
                    "url" => $Product->getUrlRewrittenWithHost($Project)
                ];
            }
        }

        $uniqueId = '';

        try {
            $User = QUI::getUsers()->get($Customer->getUUID());
            $uniqueId = $User->getUUID();
        } catch (Exception) {
        }

        // curl init
        $customer = [
            "firstName" => $Address->getAttribute('firstname'),
            "lastName" => $Address->getAttribute('lastname'),
            "email" => $email,
            "address" => $Address->getText(),
            "mobile" => $phone,
            "reference" => 'acc-' . $uniqueId
        ];

        $Date = DateTime::createFromFormat("Y-m-d H:i:s", $Order->getCreateDate());

        $transaction = [
            "reference" => $Order->getPrefixedId(),
            "date" => $Date->format(DateTimeInterface::RFC3339)
        ];

        $events = [
            "type" => "checkout",
            "defaultLocale" => $Customer->getCountry()->getLocaleCode(),
            "system" => "ecoyn",
            "systemVersion" => QUI::version(),
            "createdAt" => $Date->format(DateTimeInterface::RFC3339),
            "channel" => [
                "id" => $channel,
                "type" => "etrusted"
            ],
            "customer" => $customer,
            "transaction" => $transaction
        ];

        if (!empty($products)) {
            $events['products'] = $products;
        }


        // send
        $curl = curl_init();
        curl_setopt($curl, CURLOPT_URL, 'https://api.etrusted.com/events');
        curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($curl, CURLOPT_HEADER, false);
        curl_setopt($curl, CURLOPT_TIMEOUT, 30);
        curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, true);
        curl_setopt($curl, CURLOPT_HTTPHEADER, [
            'Content-Type: application/x-www-form-urlencoded',
            'Authorization: Bearer ' . $token
        ]);

        curl_setopt($curl, CURLOPT_POSTFIELDS, json_encode($events));

        $response = curl_exec($curl);
        $response = json_decode($response, true);
        $err = curl_error($curl);
        curl_close($curl);

        try {
            $Order->setData('quiqqer-trustedshops-send-events', 1);
            $Order->update();
        } catch (Exception $Exception) {
            QUI\System\Log::addError($Exception->getMessage());
        }
    }

    /**
     * @param Project|null $Project
     * @return mixed
     * @throws Exception
     */
    public static function getToken(?Project $Project = null): mixed
    {
        if ($Project === null) {
            $Project = QUI::getProjectManager()->get();
        }

        $key = $Project->getConfig('trustedshops.api.key');
        $clientId = $Project->getConfig('trustedshops.api.clientId');

        $curl = curl_init();
        curl_setopt($curl, CURLOPT_URL, 'https://login.etrusted.com/oauth/token');
        curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($curl, CURLOPT_HEADER, false);
        curl_setopt($curl, CURLOPT_TIMEOUT, 30);
        curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, true);
        curl_setopt($curl, CURLOPT_HTTPHEADER, [
            'Content-Type: application/x-www-form-urlencoded'
        ]);

        curl_setopt(
            $curl,
            CURLOPT_POSTFIELDS,
            http_build_query([
                'client_id' => $clientId,
                'client_secret' => $key,
                'grant_type' => 'client_credentials',
                'audience' => 'https://api.etrusted.com'
            ])
        );

        $response = curl_exec($curl);
        $response = json_decode($response, true);
        $err = curl_error($curl);
        curl_close($curl);

        if (!empty($response['access_token'])) {
            return $response['access_token'];
        }

        QUI\System\Log::addError('Trusted Shop oauth authentication failed', [
            'response' => $response,
            'curlError' => $err
        ]);

        return false;
    }

    /**
     * Return a list of all available trusted shops channels for the project
     *
     * @param Project|null $Project
     * @return array
     * @throws Exception
     */
    public static function getChannels(?Project $Project = null): array
    {
        if ($Project === null) {
            $Project = QUI::getProjectManager()->get();
        }

        $curl = curl_init();
        curl_setopt($curl, CURLOPT_URL, 'https://api.etrusted.com/channels');
        curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($curl, CURLOPT_HEADER, false);
        curl_setopt($curl, CURLOPT_TIMEOUT, 30);
        curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, true);
        curl_setopt($curl, CURLOPT_HTTPHEADER, [
            'Authorization: Bearer ' . TrustedShops::getToken($Project)
        ]);

        $response = curl_exec($curl);
        $response = json_decode($response, true);

        if (is_array($response)) {
            return $response;
        }

        return [];
    }

    /**
     * @param Project|null $Project
     * @return string
     *
     * @throws Exception
     */
    public static function getChannel(?Project $Project = null): string
    {
        if ($Project === null) {
            $Project = QUI::getProjectManager()->get();
        }

        $channel = $Project->getConfig('trustedshops.api.channel');

        if (!empty($channel)) {
            return $channel;
        }

        $channels = self::getChannels($Project);

        if (!empty($channels)) {
            return $channels[0]['id'];
        }

        throw new Exception('No trustedshops channel found');
    }
}
